/*
 * $LastChangedBy$
 * $LastChangedDate$
 * $LastChangedRevision$
 * $HeadURL$
 *
 * Copyright 2014 BlueConic Inc./BlueConic B.V. All rights reserved.
 */

public class EngagementRankingListener: BlueConicPlugin {

    private var _client: BlueConicClient!
    private var _context: InteractionContext!
    private var _engagementService: EngagementService?

    /**
     Default initializer for a blueconic plugin
     - parameter client: An instance of the BlueConicClient
     - parameter context: An instance of the InteractionContext
     */
    public required convenience init(client: BlueConicClient, context: InteractionContext) {
        self.init()
        self._client = client
        self._context = context
    }

    /**
     OnLoad function for blueconic plugin.
     This starts the plugin.
     */

    public func onLoad() {
        guard
            let interactionId = _context.getInteractionId(),
            let interestsDictionary = ListenerUtil.getDictionaryFromParameters(_context, key: PluginConstants.Listener.TAG_INTERESTS),
            let engagementRules = ListenerUtil.getDictionaryFromParameters(_context, key: PluginConstants.Listener.TAG_ENGAGEMENT_RULES)
            else {
                return
        }

        let interests: [String] = interestsDictionary[PluginConstants.Listener.TAG_VALUES] as? [String] ?? []

        // Check if there are changes in the engagementservice
        self._engagementService = EngagementService(client: self._client, listenerId: interactionId, propertyName: interactionId, isInterest: true, allInterests: interests, useHalfTime: true)
        guard let service = self._engagementService else {
            return
        }

        if let rules = engagementRules[PluginConstants.Listener.TAG_RULES] {
            service.applyRules(rules)
        }

        if service.isChanged() {
            // Store the internal JSON string or the changes to the profile
            service.save()
        }
    }

    public func onDestroy() {
        BlueConicEventFactory.getInstance().clearEventHandlers(self._context.getInteractionId())
    }

}
