/*
* $LastChangedBy$
* $LastChangedDate$
* $LastChangedRevision$
* $HeadURL$
*
* Copyright 2014 BlueConic Inc./BlueConic B.V. All rights reserved.
*/

import Foundation
class ProfileHash {
    struct Static {
        static let SEPARATOR: String = ";"
        static let M: Int = 10000
    }

    /**
    Returns the hash for all properties in a map. Entries are separated by a ';'.
    - parameter properties: the properties to get the hash for.
    - returns: a string representation for the hash
    */
    class func getHash(_ properties: [String: [String]]?) -> String {
        var result: String = ""
        if let properties = properties {
            for (key, value) in properties {
                result += getHash(key, values: value) + Static.SEPARATOR
            }
            return result
        }

        return result
    }

    /**
    Returns the hash value for a single property and its values.
    - parameter id: the profile property id
    - parameter values: the values for this property.
    - returns: a string representation for the hash
    */
    class func getHash(_ id: String, values: [String]?) -> String {
        var result = ""
        if let values = values {

            var hash = getHash(id)
            for value in values {
                hash += getHash(value)
            }
            result = "\(hash % Static.M)"
        }

        return result
    }

    /**
    Hashing method for a single string.
    - parameter input: the string to hash.
    - returns: the hash
    */
    class func getHash(_ input: String?) -> Int {
        var sum = 0
        if let input = input {
            let characters = Array(input)
            for character in characters {
                for unicodeScalar in String(character).unicodeScalars {
                    sum += Int("\(unicodeScalar.value)")!
                }
            }
        }

        return sum
    }
}
